@@ -2,13 +2,14 @@ package OpenGuides::Utils;
 
 use strict;
 use vars qw( $VERSION );
-$VERSION = '0.09';
+$VERSION = '0.11';
 
 use Carp qw( croak );
 use Wiki::Toolkit;
 use Wiki::Toolkit::Formatter::UseMod;
 use Wiki::Toolkit::Plugin::RSS::Reader;
 use URI::Escape;
+use MIME::Lite;
 
 =head1 NAME
 
@@ -328,6 +329,168 @@ sub get_wgs84_coords {
     }
 }
 
+=item B<detect_redirect>
+
+    $redir = OpenGuides::Utils->detect_redirect( content => "foo" );
+
+Checks the content of a node to see if the node is a redirect to another
+node.  If so, returns the name of the node that this one redirects to.  If
+not, returns false.
+
+(Also returns false if no content is provided.)
+
+=cut
+
+sub detect_redirect {
+    my ( $self, %args ) = @_;
+    return unless $args{content};
+
+    if ( $args{content} =~ /^#REDIRECT\s+(.+?)\s*$/ ) {
+        my $redirect = $1;
+
+        # Strip off enclosing [[ ]] in case this is an extended link.
+        $redirect =~ s/^\[\[//;
+        $redirect =~ s/\]\]\s*$//;
+
+        return $redirect;
+    }
+}
+
+=item B<validate_edit>
+
+    my $fails = OpenGuides::Utils->validate_edit(
+        id       => $node,
+        cgi_obj  => $q
+    );
+
+Checks supplied content for general validity. If anything is invalid,
+returns an array ref of errors to report to the user.
+
+=cut
+
+sub validate_edit {
+    my ( $self, %args ) = @_;
+    my $q = $args{cgi_obj};
+    my @fails;
+    push @fails, "Content missing" unless $q;
+    return \@fails if @fails;
+
+    # Now do our real validation
+    # Numeric tests
+    foreach my $var (qw(os_x os_y)) {
+        if ($q->param($var) and $q->param($var) !~ /^-?\d+$/) {
+            push @fails, "$var must be integer, was: " . $q->param($var);
+        }
+    }
+
+    foreach my $var (qw(latitude longitude)) {
+        if ($q->param($var) and $q->param($var) !~ /^-?\d+\.?(\d+)?$/) {
+            push @fails, "$var must be numeric, was: " . $q->param($var);
+        }
+    }
+
+    return \@fails;
+
+};
+
+=item B<parse_change_comment>
+
+    my $change_comment = parse_change_comment($string, $base_url);
+    
+Given a base URL (for example, C<http://example.com/wiki.cgi?>), takes a string, 
+replaces C<[[page]]> and C<[[page|titled link]]> with
+
+    <a href="http://example.com/wiki.cgi?page">page</a>
+
+and
+
+    <a href="http://example.com/wiki.cgi?page">titled link</a>
+
+respectively, and returns it. This is a limited subset of wiki markup suitable for
+use in page change comments.
+
+=cut
+
+sub parse_change_comment {   
+    my ($comment, $base_url) = @_;
+
+    my @links = $comment =~ m{\[\[(.*?)\]\]}g;
+
+    # It's not all that great having to reinvent the wheel in this way, but
+    # Text::WikiFormat won't let you specify the subset of wiki notation that 
+    # you're interested in. C'est la vie.
+    foreach (@links) {
+        if (/(.*?)\|(.*)/) {
+            my ($page, $title) = ($1, $2);
+            $comment =~ s{\[\[$page\|$title\]\]}
+                         {<a href="$base_url$page">$title</a>};
+        } else {
+            my $page = $_;
+            $comment =~ s{\[\[$page\]\]}
+                         {<a href="$base_url$page">$page</a>};
+        }
+    }
+
+    return $comment;
+}
+
+=item B<send_email>
+
+    eval { OpenGuides::Utils->send_email(
+            config        => $config,
+            subject       => "Subject",
+            body          => "Test body",
+            admin         => 1,
+            nobcc         => 1,
+            return_output => 1
+    ) };
+
+    if ($@) {
+        print "Error mailing admin: $@\n";
+    } else {
+        print "Mailed admin\n";
+    }
+
+Send out email. If C<admin> is true, the email will be sent to the site
+admin. If C<to> is defined, email will be sent to addresses in that
+arrayref. If C<nobcc> is true, there will be no Bcc to the admin.
+
+C<subject> and C<body> are mandatory arguments.
+
+Debugging: if C<return_output> is true, the message will be returned as
+a string instead of being sent by email.
+
+=cut
+
+
+sub send_email {
+    my ( $self, %args ) = @_;
+    my $config = $args{config} or die "config argument not supplied";
+    my @to;
+    @to = @{$args{to}} if $args{to};
+    my @bcc;
+    push @to, $config->contact_email if $args{admin};
+    die "No recipients specified" unless $to[0];
+    die "No subject specified" unless $args{subject};
+    die "No body specified" unless $args{body};
+    my $to_str = join ',', @to;
+    push @bcc, $config->contact_email unless $args{nobcc};
+    my $bcc_str = join ',', @bcc;
+    my $msg = MIME::Lite->new(
+        From    => $config->contact_email,
+        To      => $to_str,
+        Bcc     => $bcc_str,
+        Subject => $args{subject},
+        Data    => $args{body}
+    );
+
+    if ( $args{return_output} ) {
+        return $msg->as_string;
+    } else {
+        $msg->send or die "Couldn't send mail!";
+    }
+}
+
 =back
 
 =head1 AUTHOR
@@ -336,7 +499,7 @@ The OpenGuides Project (openguides-dev@lists.openguides.org)
 
 =head1 COPYRIGHT
 
-     Copyright (C) 2003-2007 The OpenGuides Project.  All Rights Reserved.
+     Copyright (C) 2003-2008 The OpenGuides Project.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
